#include "receiptsaveform.h"
#include "ui_receiptsaveform.h"

#include <QList>
#include <QCompleter>
#include <QSortFilterProxyModel>
#include <QPushButton>

#include <service.h>
#include <iplugin.h>
#include <addressbookservice.h>
#include <addressbookdata.h>

#include "data/voucher.h"
#include "shopservice.h"
#include "shop-odb.hxx"

ReceiptSaveForm::ReceiptSaveForm(QSharedPointer<Voucher> voucher, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ReceiptSaveForm)
{
    ui->setupUi(this);

    m_voucherModel = new AutoTableModel<Voucher>(this);
    Service<Voucher> srv;
    m_voucherModel->setData(srv.all(QString("status = %1").arg(QString::number(Voucher::NOT_PAID))));
    m_voucherModel->setTranslations(Context::instance().plugin("SHOP")->translations());
    ui->tabVouchers->setModel(m_voucherModel);
    ui->tabVouchers->hideColumn(0);
    ui->tabVouchers->hideColumn(1);
    ui->tabVouchers->hideColumn(5);
    ui->tabVouchers->hideColumn(6);
    ui->tabVouchers->hideColumn(7);
    ui->tabVouchers->hideColumn(8);
    ui->tabVouchers->hideColumn(9);
    ui->tabVouchers->hideColumn(10);
    ui->tabVouchers->hideColumn(12);
    ui->tabVouchers->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
    ui->tabVouchers->setColumnWidth(3, 200);
    ui->tabVouchers->setColumnWidth(4, 200);

    connect(ui->tabVouchers->selectionModel(), &QItemSelectionModel::currentRowChanged, [this](QModelIndex, QModelIndex){
        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
    });

    connect(ui->name, &QLineEdit::textChanged, [this](QString text){
        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!text.isEmpty());
    });

    AddressBookService srvAdb;
    QList<ComboData> comboData;
    foreach (QSharedPointer<AddressbookData> adb, srvAdb.all()) {
        comboData << ComboData(adb);
    }

    m_voucher = voucher;
    m_binder.setData(m_voucher.data());
    m_binder.registerBinding(ui->contact, comboData);
    m_binder.registerBinding(ui->name);
    m_binder.registerBinding(ui->description);
    m_binder.bindToUi();

    ui->contact->completer()->setCompletionMode(QCompleter::PopupCompletion);
    ui->contact->setCurrentIndex(-1);

    m_saveAsNew = false;
    ui->groupBox_2->setEnabled(false);

    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
}

ReceiptSaveForm::~ReceiptSaveForm()
{
    delete ui;
}

void ReceiptSaveForm::on_lineEdit_textChanged(const QString &text)
{
    QSortFilterProxyModel proxy;
    proxy.setSourceModel(m_voucherModel);
    proxy.setFilterKeyColumn(2);
    proxy.setFilterFixedString(text);

    QModelIndex matchingIndex = proxy.mapToSource(proxy.index(0,0));
    if(matchingIndex.isValid()){
        ui->tabVouchers->scrollTo(matchingIndex,QAbstractItemView::EnsureVisible);
        ui->tabVouchers->setCurrentIndex(matchingIndex);
        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
    }
}

void ReceiptSaveForm::on_radioAdd_toggled(bool checked)
{
    if (checked)
    {
        m_saveAsNew = false;
        ui->groupBox_2->setEnabled(false);
        ui->groupBox->setEnabled(true);

        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(ui->tabVouchers->currentIndex().isValid());
    }
    else
    {
        m_saveAsNew = true;
        ui->groupBox_2->setEnabled(true);
        ui->groupBox->setEnabled(false);

        ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!ui->name->text().isEmpty());
    }
}

void ReceiptSaveForm::accept()
{
    ShopService srv;
    m_binder.bindToData();
    if (m_saveAsNew)
    {
        m_voucher->setStatus(Voucher::NOT_PAID);
        srv.updateVoucher(m_voucher);
    }
    else
    {
        QSharedPointer<Voucher> voucher = m_voucherModel->itemFromIndex(ui->tabVouchers->currentIndex());
        foreach (QSharedPointer<VoucherItem> item, m_voucher->items()) {
            voucher->addItem(item);
        }

        srv.calculate(voucher);
        srv.updateVoucher(voucher);
        srv.erase(m_voucher);
    }

    QDialog::accept();
}
